#include "mex.h"

/*   MAXEIG Largest eigenvalue of symmetric tridiagonal matrix.   */

/*   P.-O. Persson <persson@mit.edu>   */
/*   Department of Mathematics, MIT    */
/*   November 12, 2002                 */

#define DBL sizeof(double)
#define INT sizeof(int)

void mexFunction( int nlhs, mxArray *plhs[], 
		  int nrhs, const mxArray*prhs[] )
{
  double *D,*E;
  size_t m,n,info,nsplit;
  double z=0.0;
  char *tmp;

  if (nrhs!=2)
    mexErrMsgTxt("Must have exactly two input arguments.");
  if (nlhs>1)
    mexErrMsgTxt("Too many output arguments.");
  if (!mxIsDouble(prhs[0]) || !mxIsDouble(prhs[1]))
    mexErrMsgTxt("Both inputs must be double arrays.");
  if (mxIsComplex(prhs[0]) || mxIsComplex(prhs[1]))
    mexErrMsgTxt("Both inputs must be real.");
  if (mxGetM(prhs[0])!=1 && mxGetN(prhs[0])!=1)
    mexErrMsgTxt("First input must be a vector.");
  if (mxGetM(prhs[1])!=1 && mxGetN(prhs[1])!=1)
    mexErrMsgTxt("Second input must be a vector.");

  n=mxGetNumberOfElements(prhs[0]);

  if (mxGetNumberOfElements(prhs[1])!=n-1)
    mexErrMsgTxt("The input vectors must have length N and N-1.");

  D=mxGetPr(prhs[0]);
  E=mxGetPr(prhs[1]);

  tmp=(char*)mxMalloc(5*n*DBL+5*n*INT);

  /* Remove _ on Windows */
  dstebz_("I","B",&n,0,0,&n,&n,&z,D,E,&m,&nsplit,
          tmp,tmp+n*DBL,tmp+n*DBL+n*INT,
          tmp+n*DBL+2*n*INT,tmp+n*DBL+2*n*INT+4*n*DBL,
          &info);

  plhs[0]=mxCreateScalarDouble(((double*)tmp)[0]);
  mxFree(tmp);
  if (info)
    mexErrMsgTxt("Error in DSTEBZ.");
}
